local libInfo = 'FLASHDRIVE 0.3 - 28.04.2018'

includeParentLib('ROOT_DEVICE.lua')

-- Definitions for Malfunction types

device.fail.pcb = device.malfunctionClass:new('PCB failure') 
device.fail.fw = device.malfunctionClass:new('FW damage') 
device.fail.memory = device.malfunctionClass:new('bad memory cells')

-- Rules for processing information collected from the form  =========================================================================

device.diag[regFunction('NOTDETECTED_ELECTRICAL_DAMAGES', libInfo)] = function () 
    if  device.FORM_ELECTRICAL_DAMAGE == 'YES' 
	or device.FORM_ABNORMAL_HEAT == 'YES' 
	or device.FORM_PCB_DAMAGE == 'YES' 
    then 
        device.fail.pcb:probability()
        addExplanation('Questionnaire data suggests that PCB has failed.')
    end
	
end

-- Rules for processing test results =================================================================================================

device.diag[regFunction('3D_READ', libInfo)] = function ()

    if device.testReadOD and device.testReadOD.status == 'FAIL'
    or device.testReadMD and device.testReadMD.status == 'FAIL'
    or device.testReadID and device.testReadID.status == 'FAIL'
    then 
        device.fail.memory:probability()
        addExplanation('During the test some memory areas were found to be unreadable.')
	end

    if  device.testReadOD and device.testReadOD.status == 'PASS'
    and device.testReadMD and device.testReadMD.status == 'FAIL'
    and device.testReadID and device.testReadID.status == 'FAIL'
    then 
        device.fail.fw:probability()
        addExplanation('Read errors emerging after a successful read of the beginning of the address space may indicate translator damage.')
    elseif device.testReadOD and device.testReadOD.status == 'PASS'
    and device.testReadMD and device.testReadMD.status == 'PASS'
    and device.testReadID and device.testReadID.status == 'FAIL'
    then 
        device.fail.fw:probability()
        addExplanation('Read errors emerging after a successful read of the beginning of the address space may indicate translator damage.')
	end
   
end

device.diag[regFunction('RANDOM_READ', libInfo)] = function ()

    if device.testRandomRead and device.testRandomRead.status == 'FAIL'
    then
        device.fail.memory:probability()
        addExplanation('During the test some memory areas were found to be unreadable.')
    end
	
end

device.diag[regFunction('CHECK_CAPACITY', libInfo)] = function ()

    if device.CAPACITY_USER_ASSUMED and device.CAPACITY_PASSPORT/device.CAPACITY_USER_ASSUMED < 0.3
    then
        device.fail.pcb:probability()
		device.fail.fw:probability()
        addExplanation('Capacity returned by the device is a fraction of the user-specified capacity. This may indicate either a physically damaged controller or a firmware damage.')
    end
	
end