local libInfo = 'HDD 0.2 - 05.03.2018'

includeParentLib('ROOT_DEVICE.lua')

-- Definitions for Malfunction types

device.fail.pcb = device.malfunctionClass:new('PCB malfunction') 
device.fail.fw = device.malfunctionClass:new('FW corruption') 
device.fail.jam = device.malfunctionClass:new('spindle jammed') 
device.fail.heads = device.malfunctionClass:new('defective head(s)')
device.fail.bb = device.malfunctionClass:new('bad blocks')
device.fail.scratch = device.malfunctionClass:new('scratched surface')

-- SCSI 
device.fail.writefault = device.malfunctionClass:new('write faults')
device.fail.transator = device.malfunctionClass:new('translator issue')

-- Rules for processing test results =================================================================================================

device.diag[regFunction('3D_READ', libInfo)] = function ()

-- Heads ---------------------------------- 
    if  device.testReadOD and device.testReadOD.status == 'FAIL'
    and device.testReadMD and device.testReadMD.status == 'FAIL'
    and device.testReadID and device.testReadID.status == 'FAIL'
    then 
        device.fail.heads:probability()
        device.fail.bb:highProbability()
        addExplanation('Read attempts failed at the beginning, middle and at the end of the drive, which may be a sign of bad heads.')

-- BB ----------------------------------   
    elseif device.testReadOD and device.testReadOD.status == 'FAIL' and device.testReadOD.errCount > 1
    or device.testReadMD and device.testReadMD.status == 'FAIL' and device.testReadMD.errCount > 1
    or device.testReadID and device.testReadID.status == 'FAIL' and device.testReadID.errCount > 1
    then 
        device.fail.bb:highProbability()
        addExplanation('During the test a large number of sectors were found to be read incorrectly.')
   
    elseif device.testReadOD and device.testReadOD.status == 'FAIL' and device.testReadOD.errCount == 1
    or device.testReadMD and device.testReadMD.status == 'FAIL' and device.testReadMD.errCount == 1
    or device.testReadID and device.testReadID.status == 'FAIL' and device.testReadID.errCount == 1
    then 
        device.fail.bb:probability()
        addExplanation('During the test some sectors were found to be read incorrectly.')

    elseif device.testReadOD and device.testReadOD.status == 'PASS' and device.testReadOD.errCount > 0
    or device.testReadMD and device.testReadMD.status == 'PASS' and device.testReadMD.errCount > 0
    or device.testReadID and device.testReadID.status == 'PASS' and device.testReadID.errCount > 0
    then 
        device.fail.bb:probability()
        addExplanation('During the test some sectors were found to be read incorrectly.')
		
    end
   
end

device.diag[regFunction('RANDOM_READ', libInfo)] = function ()

    if device.testRandomRead and device.testRandomRead.status == 'FAIL'
    then
        device.fail.bb:highProbability()
        addExplanation('During the test a large number of sectors were found to be read incorrectly.')
    end
	
end



-- Rules for UNDETECTED drives=================================================================================================


--BZZZ-----------------------------------------------------------------------------------------------------------------------------
device.diag[regFunction('NOTDETECTED_BZZZ', libInfo)] = function () 
    if  device.NOTDETECTED 
    and device.FORM_SOUND == 'BZZZ' 
    and device.FORM_SPINS_UP ~= 'YES' 
    then 
        device.fail.jam:highProbability()
        addExplanation('Undetected drive making buzzing/humming noise is frequently a sign of a spindle jam.')
    end
end

--SKIRR
device.diag[regFunction('NOTDETECTED_SKIRR', libInfo)] = function ()    
    if  device.NOTDETECTED 
    and device.FORM_SOUND == 'SKIRR' 
    and device.FORM_SPINS_UP ~= 'NO' 
    then 
        device.fail.scratch:highProbability()
        addExplanation('Undetected drive making scratching/skirring sound is frequently a sign of a surface scratch.')
    end
end

-- KNOCKS-------------------------------------------------------------------------------------------------------------------------------------------------------
device.diag[regFunction('NOTDETECTED_KNOCKS', libInfo)] = function ()   
    if  device.NOTDETECTED 
    and device.FORM_SOUND == 'KNOCKS' 
    and device.FORM_SPINS_UP ~= 'NO' 
    then 
        if  device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        and device.FORM_MECHANICAL_SHOCK ~= 'YES'
        and device.FORM_ROM_NOT_ORIGINAL ~= 'YES'
        and device.FORM_PCB_DAMAGE ~= 'YES'
        and device.FORM_SMELLS_BURNT ~= 'YES'
        then      
            device.fail.heads:probability()
            device.fail.bb:probability()
            device.fail.pcb:lowProbability()
            device.fail.scratch:lowProbability()
            addExplanation('Undetected drive making clicking/knocking sound is frequently a sign of bad heads and/or surface, and rarely means PCB issues or surface.')
        end
           
        if  device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        and device.FORM_ROM_NOT_ORIGINAL == 'YES'
        then
            device.fail.fw:probability()
            addExplanation('Clicking/knocking sound out of a drive with non-original ROM may be a sign that firmware is not compatible.')
        end

        if  ( device.FORM_ELECTRICAL_DAMAGE == 'YES' or  device.FORM_PCB_DAMAGE == 'YES' or device.FORM_SMELLS_BURNT == 'YES' ) 
        then
            device.fail.pcb:highProbability()
            device.fail.heads:probability()
            device.fail.scratch:lowProbability()
            device.fail.bb:lowProbability()
            addExplanation('Clicking/knocking sound out of a drive with electrical damage or damaged PCB or burnt smell is frequently a sign of PCB issues, but may also indicate bad heads and sometimes bad blocks and surface damage.')
        end

        if  device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        and device.FORM_PCB_DAMAGE ~= 'YES'
        and device.FORM_MECHANICAL_SHOCK == 'YES'
        and device.FORM_ROM_NOT_ORIGINAL ~= 'YES'
        then
            device.fail.heads:highProbability()
            device.fail.scratch:probability()
            device.fail.bb:probability()
            addExplanation('Clicking/knocking sound out of a drive after a physical shock usually means dead heads, frequently involving scratches on the platter surface (head crash), or may indicate bad blocks.')
        end
    end
end

--NORMAL----------------------------------------------------------------------------------------------------------------------------------------
device.diag[regFunction('NOTDETECTED_NORMAL', libInfo)] = function ()   
    if device.NOTDETECTED 
    and device.FORM_SOUND == 'NORMAL' 
    and device.FORM_SPINS_UP ~= 'NO' 
    then 
        if  device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        and device.FORM_MECHANICAL_SHOCK ~= 'YES'
        and device.FORM_ROM_NOT_ORIGINAL ~= 'YES'
        and device.FORM_PCB_DAMAGE ~= 'YES' 
        and device.FORM_SMELLS_BURNT ~= 'YES'
        then 
            device.fail.fw:highProbability()
            device.fail.bb:lowProbability()
            device.fail.heads:lowProbability()
            device.fail.pcb:lowProbability()
            addExplanation('Regular sound coupled with non-detection issues frequently means firmware damage and sometimes indicates either bad blocks or PCB issues or bad heads.')
        end

        if  device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        and device.FORM_MECHANICAL_SHOCK == 'YES'
        and device.FORM_ROM_NOT_ORIGINAL ~= 'YES'
        and device.FORM_PCB_DAMAGE ~= 'YES' 
        and device.FORM_SMELLS_BURNT ~= 'YES'
        then 
            device.fail.fw:highProbability()
            device.fail.bb:highProbability()
            device.fail.heads:probability()
            device.fail.scratch:lowProbability()
            addExplanation('Regular sound out of an undetected drive after a physical shock frequently means firmware damage and bad blocks, but may also indicate a surface scratch or bad heads.')
        end

        if  device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        and device.FORM_MECHANICAL_SHOCK ~= 'YES'
        and device.FORM_ROM_NOT_ORIGINAL == 'YES'
        and device.FORM_PCB_DAMAGE ~= 'YES' 
        and device.FORM_SMELLS_BURNT ~= 'YES'
        then 
            device.fail.fw:highProbability()
            device.fail.bb:lowProbability()
            device.fail.heads:lowProbability()
            device.fail.pcb:probability()
            addExplanation('Regular sound out of an undetected drive with non-original ROM frequently means firmware incompatibility or PCB issues and sometimes indicates bad blocks or bad heads.')
        end
    end
end

--SILENCE---------------------------------------------------------------------------------------------------------------------------------------------
device.diag[regFunction('NOTDETECTED_SILENCE', libInfo)] = function ()  
    if  device.NOTDETECTED 
    and device.FORM_SOUND == 'SILENCE' 
    and device.FORM_SPINS_UP ~= 'YES' 
    then   
        if  device.FORM_ROM_NOT_ORIGINAL == 'YES'
        and device.FORM_MECHANICAL_SHOCK ~= 'YES'
        and device.FORM_PCB_DAMAGE ~= 'YES'
        and device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        then
            device.fail.fw:highProbability()
            addExplanation('No sound out of a drive with non-original ROM usually means incompatible firmware.')
        end
      
        if  device.FORM_ELECTRICAL_DAMAGE ~= 'YES'
        and device.FORM_PCB_DAMAGE ~= 'YES'
        and device.FORM_ROM_NOT_ORIGINAL ~= 'YES'
        and device.FORM_SMELLS_BURNT ~= 'YES'
        then
            device.fail.pcb:probability()
            device.fail.heads:probability()
            addExplanation('No sound out of a drive in its reference operating condition is a sign of PCB issues or bad heads.')
        end
 
        if  ( device.FORM_ELECTRICAL_DAMAGE == 'YES' or  device.FORM_SMELLS_BURNT == 'YES' )
        and device.FORM_PCB_DAMAGE ~= 'YES'
        and device.FORM_MECHANICAL_SHOCK ~= 'YES'
        and device.FORM_ROM_NOT_ORIGINAL ~= 'YES'
        then
            device.fail.pcb:highProbability()
            device.fail.heads:probability()
            addExplanation('No sound out of a drive with suspected PCB failure is frequently a confirmation of PCB issues and may also be a sign of bad heads.')
        end
    end    
end