local libInfo = 'SSD 0.1 - 28.04.2108'

includeParentLib('ROOT_DEVICE.lua')

-- Definitions for Malfunction types

device.fail.pcb = device.malfunctionClass:new('PCB failure') 
device.fail.fw = device.malfunctionClass:new('FW damage')
device.fail.bb = device.malfunctionClass:new('bad blocks')

-- Rules for processing test results =================================================================================================

device.diag[regFunction('3D_READ', libInfo)] = function ()

    if  device.testReadOD and device.testReadOD.status == 'FAIL'
    and device.testReadMD and device.testReadMD.status == 'FAIL'
    and device.testReadID and device.testReadID.status == 'FAIL'
    then 
        device.fail.bb:highProbability()
        addExplanation('Read attempts failed at the beginning, middle and at the end of the drive, which may be a sign of bad NAND pages.')

    elseif device.testReadOD and device.testReadOD.status == 'FAIL' and device.testReadOD.errCount > 1
    or device.testReadMD and device.testReadMD.status == 'FAIL' and device.testReadMD.errCount > 1
    or device.testReadID and device.testReadID.status == 'FAIL' and device.testReadID.errCount > 1
    then 
        device.fail.bb:highProbability()
        addExplanation('During the test a large number of sectors were found to be read incorrectly.')
   
    elseif device.testReadOD and device.testReadOD.status == 'FAIL' and device.testReadOD.errCount == 1
    or device.testReadMD and device.testReadMD.status == 'FAIL' and device.testReadMD.errCount == 1
    or device.testReadID and device.testReadID.status == 'FAIL' and device.testReadID.errCount == 1
    then 
        device.fail.bb:probability()
        addExplanation('During the test some sectors were found to be read incorrectly.')

    elseif device.testReadOD and device.testReadOD.status == 'PASS' and device.testReadOD.errCount > 0
    or device.testReadMD and device.testReadMD.status == 'PASS' and device.testReadMD.errCount > 0
    or device.testReadID and device.testReadID.status == 'PASS' and device.testReadID.errCount > 0
    then 
        device.fail.bb:probability()
        addExplanation('During the test some sectors were found to be read incorrectly.')
		
    end
   
end

device.diag[regFunction('RANDOM_READ', libInfo)] = function ()

    if device.testRandomRead and device.testRandomRead.status == 'FAIL'
    then
        device.fail.bb:highProbability()
        addExplanation('During the test a large number of sectors were found to be read incorrectly.')
    end
	
end



-- Rules for UNDETECTED drives=================================================================================================

--Electrical damages-----------------------------------------------------------------------------------------------------------------------------
device.diag[regFunction('NOTDETECTED_ELECTRICAL_DAMAGES', libInfo)] = function () 
    if  device.NOTDETECTED 
    and device.FORM_ELECTRICAL_DAMAGE == 'YES' 
	or device.FORM_ABNORMAL_HEAT == 'YES' 
	or device.FORM_PCB_DAMAGE == 'YES' 
    then 
        device.fail.pcb:highProbability()
        addExplanation('As you can see, the device has failed.')
    end
	
	if  device.NOTDETECTED 
    and device.FORM_ELECTRICAL_DAMAGE == 'NO' 
	and device.FORM_ABNORMAL_HEAT == 'NO' 
	and device.FORM_PCB_DAMAGE == 'NO' 
    then 
        device.fail.FW:highProbability()
		device.fail.pcb:Probability()
        addExplanation('70% probability of firmware damage and 30% probability of PCB damage.')
    end
	
end


