local libInfo = 'SSD_ATA - 28.04.2018'

includeParentLib('SSD.lua')

-- Rules for processing test results =================================================================================================

device.diag[regFunction('RANDOM_READ', libInfo)] = function ()

    if device.testRandomRead and device.testRandomRead.status == 'FAIL'
    then
        device.fail.bb:highProbability()
        addExplanation('During the test a large number of blocks were found to be read incorrectly.')
    end
end


-- Rules for the S.M.A.R.T analisys. =================================================================================================

-- Malfunctions-----------------------------------------------------------------------------------------------------------------------------------------
device.diag[regFunction('SMART_DIAGNOSIS', libInfo)] = function ()
-- #5 and 198
    if device.attr5 and ( device.attr5.raw1 > 499 or device.attr5.value < 26 ) 
    or device.attr198 and ( device.attr198.raw1 > 499 or device.attr198.value < 26 )    
    then 
        device.fail.bb:probability()
        addExplanation('According to the S.M.A.R.T. attributes analysis, the drive is likely to have unreadable sectors.')
    end
-- #184
    if device.attr184 and device.attr184.raw1 > 0 
    then
        device.fail.pcb:lowProbability()
        addExplanation('A non-zero value of the attribute 184 may indicate a PCB or controller problem, poor contact or cable, or a presence of strong electromagnetic interference (EMI).') 
    end
-- #199 
    if device.attr199 and device.attr199.raw1 > 0 
    then
        device.fail.pcb:lowProbability()
        addExplanation('A non-zero value of the attribute 199 may indicate either a poor cable or a buffer memory or firmware error.')   
    end
    
    if device.attr197 and device.attr197.raw1 > 0 
    then
        if device.attr197.raw1 < 10 
        then
            device.fail.bb:probability()
            addExplanation('A non-zero value of the attribute 197 may indicate a presence of bad sectors.')    
        else
            device.fail.bb:highProbability()
            addExplanation('A non-zero value of the attribute 197 indicates a presence of bad sectors.')    
        end
    end
end

--Warnings-----------------------------------------------------------------------------------------------------------------------------------------
device.diag[regFunction('SMART_WARNINGS', libInfo)] = function ()

-- #5 and 198
    if device.attr5 and ( 0 < device.attr5.raw1 and device.attr5.raw1 < 500 or 25 < device.attr5.value and device.attr5.value < 75 )
    or device.attr198 and ( 0 < device.attr198.raw1 and device.attr198.raw1 < 500 or 25 < device.attr198.value and device.attr198.value < 75 )  
    then 
        addWarning('S.M.A.R.T. indicates signs of drive wear.')
    end
-- #5 and 196 and 197
    if device.attr5 and device.attr5.raw1 == 0 
    and ( device.attr196 and device.attr196.raw1 > 99 or  device.attr197 and device.attr197.raw1 > 99 )
    then 
        addWarning('S.M.A.R.T. indicates signs of drive wear.')
    end

-- #183
    if device.attr183 and device.attr183.raw1 > 5
    then
        addWarning('There could be a cable or controller problem.')
    end
-- #190 and 194 and 231
    if device.attr190 and device.attr190.raw1 > 50 
    or device.attr194 and device.attr194.raw1 > 50
    or device.attr231 and device.attr231.raw1 > 50
    then 
        addWarning('Drive temperature is too high.')
    end

end

