
rl = require('rl')

debugInfo = ''
diagnosis = '\n'
explanation = '\n'
warning = '\n'
resultString = ''

device = {}
device.diag = {} -- Registered rights functions
device.fail = {} -- List of mailfunction types
device.regRulesList = {} -- List of registered rights functions

function addExplanation(newExplanation) explanation = explanation .. '  ' .. rl.loc(newExplanation) .. '\n' end

function addWarning(newWarning) warning = warning .. ' ! ' .. rl.loc(newWarning) .. '\n' end

function addDebugInfo(newDebugInfo) 
    if device.DEBUG then 
        if newDebugInfo then
            debugInfo = debugInfo .. newDebugInfo .. '\n'
        else
            debugInfo = debugInfo .. '\n'
        end
    end 
end

function includeParentLib(fileName) dofile(libFolder .. fileName) end

-- Diagnostic functions registration
function regFunction(functionName, libInfo)
    if device.regRulesList[functionName] then
        addDebugInfo('Loaded ' .. functionName .. ' function from ' .. libInfo .. ' library, function from ' .. device.regRulesList[functionName] .. ' has been overloaded.')
    else
        addDebugInfo('Loaded ' .. functionName .. ' function from library ' .. libInfo .. '.')
    end
    device.regRulesList[functionName] = libInfo
    return functionName
end

-- SCSI errors decoding
function ScsiSenseDecode(errCode)
    local keySense
    local codeSense
    if not errCode then return nil, 'ErrCod is nil' end
    if errCode ~= '00000000' then
        rl.includeOnce(libFolder .. 'table_fakeSense.lua')
        if fakeSense[errCode] then 
            return fakeSense[errCode]
        else
            local key, code, qualifier, FRU = string.match(errCode, '(%w%w)(%w%w)(%w%w)(%w%w)')
            if code then
                rl.includeOnce(libFolder .. 'table_scsiSense.lua')
                keySense = scsiSense[key]
                if not keySense then return nil, 'Can\'t find sense key in SCSISense table' end
            
                codeSense, errorMessage = rl.iniVal(device.BASE_DIR .. 'SCSI_Err.ini', 'SENSE', code .. qualifier)
                if not codeSense then return nil, errorMessage end
            else return nil, 'Can\'t parse SenseCode.' end
        end
    else return 'No SCSI errors.' end
    
    return keySense .. '    ' .. codeSense
end

-- GO!!! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

 -- Tags string deserialization.
for key, value in string.gmatch(QuickDiagResult,'([^\n:]+):([^\n]+)') do
    if value == value:match('%d+') then
        device[key] = tonumber(value)
    else
        device[key] = value
    end
end

addDebugInfo(QuickDiagResult)

-- Adding missing source tags
device.FWVER = device.FWVER or 'NOTDETECTED'
device.MODEL = device.MODEL or 'NOTDETECTED'
device.MFGBRAND = device.MFGBRAND or 'NOTDETECTED'
device.FAMILY = device.FAMILY or 'GNRC'
device.FAMILY_ID = device.FAMILY_ID or 0
device.FAMILY_GEN = device.FAMILY_GEN or 'GNRC'

-- End the program if there is an error on the previous stages of diagnostics
if device.STAGE_ERROR then return device.STAGE_ERROR end

-- Assigning values to auxiliary variables
libFolder = device.BASE_DIR .. 'Scripts/lib/'

-- Fill in the values of the derivate tags ======================================================================================

-- device.CAPACITY_USER_ASSUMED assumed by user device size in sectors
if device.CAPACITY_PASSPORT and device.LOGICAL_SECTOR and device.FORM_CAPACITY and tonumber(device.FORM_CAPACITY) > 0 and device.FORM_CAPACITY_UNITS then
	    local mult
		if device.FORM_CAPACITY_UNITS == 'Mb' then mult = 1000000
		elseif device.FORM_CAPACITY_UNITS == 'Gb' then mult = 1000000000
		elseif device.FORM_CAPACITY_UNITS == 'Tb' then mult = 1000000000000
		end
		
        device.CAPACITY_USER_ASSUMED = device.FORM_CAPACITY*mult/device.LOGICAL_SECTOR
		addDebugInfo('CAPACITY_USER_ASSUMED=>' .. device.CAPACITY_USER_ASSUMED)
end


-- Refining the model name
do
    if device.PROFILE_TYPE == 'HDD_ATA' or device.PROFILE_TYPE == 'SSD_ATA' then 
        device.RMODEL =  string.match(device.MODEL,'%w+%s+(%w+-*%w*)')
    end

    if not device.RMODEL then device.RMODEL = device.MODEL end
end

-- Filling the code name of the product family
if (device.PROFILE_TYPE == 'HDD_ATA' or device.PROFILE_TYPE == 'SSD_ATA') and device.FAMILY_ID ~= 0 then

    if device.MFGBRAND == 'SAMSUNG' then
    
        rl.includeChunk('Scripts/lib/table_family2ID_SAMSUNG.lua')
        for family, id in pairs(family2ID) do

            if id == device.FAMILY_ID then 
                device.FAMILY = family
                break
            end
        end
        
    elseif device.MFGBRAND == 'HGST' then

        rl.includeChunk('Scripts/lib/table_family2ID_HGST.lua')
        for family, id in pairs(family2ID) do

            if id == device.FAMILY_ID then 
                device.FAMILY = family
                break
            end
        end
        
    elseif device.MFGBRAND == 'WDC' then

        device.FAMILY = 'F' .. device.FAMILY_ID
            
    elseif device.MFGBRAND == 'SEAGATE' then

        device.FAMILY = 'F' .. device.FAMILY_ID
        
    else end    
end

-- If Debug is enabled then send  additional info to output
addDebugInfo('RMODEL=>' .. device.RMODEL)
addDebugInfo('FAMILY=>' .. device.FAMILY)
addDebugInfo('FAMILY_GEN=>' .. device.FAMILY_GEN)

-- preliminary cooking of the tag contents ==================================================================================================================


-- Splitting string of SMART attributes for ATA devices-------------------------------------------------------------------------------------------------------------------------------------------
if device.PROFILE_TYPE == 'HDD_ATA' or device.PROFILE_TYPE == 'SSD_ATA' then
    local tmpTable = {}
    for key, value in pairs(device) do
        local SMART_ID = string.match(key,'^ATTR_(%d+)')
        if SMART_ID then
            tmpTable['attr' .. SMART_ID] = {}
            tmpTable['attr' .. SMART_ID].type, tmpTable['attr' .. SMART_ID].value, tmpTable['attr' .. SMART_ID].worst, tmpTable['attr' .. SMART_ID].threshold, tmpTable['attr' .. SMART_ID].raw1, tmpTable['attr' .. SMART_ID].raw2, tmpTable['attr' .. SMART_ID].raw3 = string.match(value,'^(%d+),(%d+),(%d+),(%d+),(%d+)/*(%d*)/*(%d*)$')
            for attrType, attrValue in pairs(tmpTable['attr' .. SMART_ID]) do
                if attrValue and attrValue ~= '' then
                    tmpTable['attr' .. SMART_ID][attrType] = tonumber(attrValue)
                end
            end
        end
    end
    for key, value in pairs(tmpTable) do device[key] = value end
end

-- Splitting strings of LOG attributes for SCSI devices-------------------------------------------------------------------------------------------------------------------------------------------
if device.PROFILE_TYPE == 'HDD_SCSI' or device.PROFILE_TYPE == 'SSD_SCSI' then
    local tmpTable = {}
    for key, value in pairs(device) do
        local logId, pId = string.match(key,'^LOG_(%w+)_(%w+)')
        if logId then
            local logName = 'log' .. logId
            device[logName] = device[logName] or {}
            device[logName]['param' .. pId] = value
        end
    end
end

-- Parsing read test results-----------------------------------------------------------------------------------------------------------------------------------------------------------------

if device.TEST_RANDOM_SEEK then
    device.testRandomSeek = {}
    device.testRandomSeek.status, device.testRandomSeek.startLBA, device.testRandomSeek.stopLBA, device.testRandomSeek.lastLBA, device.testRandomSeek.seekCount, device.testRandomSeek.seekDone, device.testRandomSeek.errCode, device.testRandomSeek.cmdTimeMin, device.testRandomSeek.cmdTimeMax, device.testRandomSeek.cmdTimeAvg = string.match(device.TEST_RANDOM_SEEK,'^(%w+),(%d+),(%d+),(%-?%d+),(%d+),(%d+),(%w+),(%d+),(%d+),(%d+)$')
    if device.testRandomSeek.status then 
        device.testRandomSeek.startLBA = tonumber(device.testRandomSeek.startLBA); device.testRandomSeek.stopLBA = tonumber(device.testRandomSeek.stopLBA); device.testRandomSeek.lastLBA = tonumber(device.testRandomSeek.lastLBA); device.testRandomSeek.seekCount = tonumber(device.testRandomSeek.seekCount); device.testRandomSeek.seekDone = tonumber(device.testRandomSeek.seekDone); device.testRandomSeek.cmdTimeMin = tonumber(device.testRandomSeek.cmdTimeMin); device.testRandomSeek.cmdTimeMax = tonumber(device.testRandomSeek.cmdTimeMax); device.testRandomSeek.cmdTimeAvg = tonumber(device.testRandomSeek.cmdTimeAvg)
    end 
    if device.DEBUG then
        addDebugInfo('\nExpanded RandomSeek test results:')
        for key, value in rl.pairs(device.testRandomSeek) do
            addDebugInfo('TestRandomSeek.' .. key .. ' = ' .. value)
        end
        local scsiSenseInfo, errorMessage = ScsiSenseDecode(device.testRandomSeek.errCode)
        if scsiSenseInfo then addDebugInfo('Last SCSI Sense:   ' .. scsiSenseInfo)
        else addDebugInfo('ERROR decoding the last error code:   '.. errorMessage) end
    end
end

if device.TEST_RANDOM_READ then
    device.testRandomRead = {}
    device.testRandomRead.status, device.testRandomRead.startLBA, device.testRandomRead.stopLBA, device.testRandomRead.lastLBA, device.testRandomRead.blockSize, device.testRandomRead.readsCount, device.testRandomRead.readsDone, device.testRandomRead.errCode, device.testRandomRead.cmdTime, device.testRandomRead.testTimeMin, device.testRandomRead.testTimeMax, device.testRandomRead.testTimeAvg = string.match(device.TEST_RANDOM_READ,'^(%w+),(%d+),(%d+),(%-?%d+),(%d+),(%d+),(%d+),(%w+),(%d+),(%d+),(%d+),(%d+)$')
    if device.testRandomRead.status then 
        device.testRandomRead.startLBA = tonumber(device.testRandomRead.startLBA); device.testRandomRead.stopLBA = tonumber(device.testRandomRead.stopLBA); device.testRandomRead.lastLBA = tonumber(device.testRandomRead.lastLBA); device.testRandomRead.blockSize = tonumber(device.testRandomRead.blockSize); device.testRandomRead.readsCount = tonumber(device.testRandomRead.readsCount); device.testRandomRead.readsDone = tonumber(device.testRandomRead.readsDone); device.testRandomRead.cmdTime = tonumber(device.testRandomRead.cmdTime); device.testRandomRead.testTimeMin = tonumber(device.testRandomRead.testTimeMin); device.testRandomRead.testTimeMax = tonumber(device.testRandomRead.testTimeMax); device.testRandomRead.testTimeAvg = tonumber(device.testRandomRead.testTimeAvg)
    end
    if device.DEBUG then
        addDebugInfo('\nExpanded RandomRead test results:')
        for key, value in rl.pairs(device.testRandomRead) do
            addDebugInfo('TestRandomRead ' .. key .. ' = ' .. value)
        end
        local scsiSenseInfo, errorMessage = ScsiSenseDecode(device.testRandomRead.errCode)
        if scsiSenseInfo then addDebugInfo('Last SCSI Sense:   ' .. scsiSenseInfo)
        else addDebugInfo('ERROR decoding the last error code:   '.. errorMessage) end
    end
end

function dddSplit(testName, tagVal)
    if tagVal then
        local table = {}
        table.status, table.startLBA, table.stopLBA, table.blockSize, table.errCount, table.errCode, table.testTime = string.match(tagVal,'^(%w+),(%d+),(%d+),(%d+),(%d+),(%w+),(%d+)$')
        if table.status then 
            table.startLBA = tonumber(table.startLBA); table.stopLBA = tonumber(table.stopLBA); table.blockSize = tonumber(table.blockSize); table.errCount = tonumber(table.errCount); table.testTime = tonumber(table.testTime)
        end
        if device.DEBUG then
            addDebugInfo('\nExpanded ' .. testName ..' test results:')
            for key, value in rl.pairs(table) do
                addDebugInfo(testName .. ' ' .. key .. ' = ' .. value)
            end
            local scsiSenseInfo, errorMessage = ScsiSenseDecode(table.errCode)
            if scsiSenseInfo then addDebugInfo('Last SCSI Sense:   ' .. scsiSenseInfo)
            else addDebugInfo('ERROR decoding the last error code:   '.. errorMessage) end
        end
    return table
    end
end 

device.testReadOD = dddSplit('TestReadOD', device.TEST_READ_OD)
device.testReadMD = dddSplit('TestReadMD', device.TEST_READ_MD)
device.testReadID = dddSplit('TestReadID', device.TEST_READ_ID)

addDebugInfo()


-- Diagnosis based on the information provided ====================================================================================================

--  Find and load the appropriate library---------------------------------------------------------------------------------------------------------------------------
do
    function libExists(libName)
        local fileName = libFolder .. libName .. '.lua'
        if rl.fileExists(fileName) then return fileName else return false end
    end

    libFileName = libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND  .. '_' .. device.RMODEL .. '_' .. device.FWVER) 
                or libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND  .. '_' .. device.RMODEL) 
--              or libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND  .. '_' .. device.FAMILY) 
                or libExists(device.PROFILE_TYPE .. '_' .. device.MFGBRAND) 
                or libExists(device.PROFILE_TYPE)
                
    if not libFileName then 
        errorMessage = 'Can\'t found any lib file for this device.\n\n'
        return errorMessage .. '\n\n' .. resultString .. debugInfo
    else 
        if device.DEBUG then resultString = 'Loaded ' .. libFileName .. ' library file \n' .. resultString  end
    end
end

local success, errorMessage = rl.protectedLoad(libFileName)

-- Perform diagnostic functions ---------------------------------------------------------------------------------------------------------------------------

if success then
    addDebugInfo()
    for diagRuleName in pairs(device.diag) do 
	globalVariableForNameDiagRuleApplied = diagRuleName
        success, errorMessage = pcall(device.diag[diagRuleName])
        if success then
            addDebugInfo('Executed ' .. diagRuleName .. ' function from '.. device.regRulesList[diagRuleName] .. ' library.')
        else 
            return 'Error in the ' ..  diagRuleName .. ' function from '.. device.regRulesList[diagRuleName] .. ' library:\n' .. errorMessage .. '\n\n' .. resultString .. debugInfo
        end
    end
else
    return errorMessage .. '\n\n' .. resultString .. debugInfo
end

-- Calling malfunction description methods ---------------------------------------------------------------------------------------------------------------------------

local malfunctionDetected = false
for diagResultName in pairs(device.fail) do
    local verdict = device.fail[diagResultName]:verdict()
    if verdict then diagnosis = diagnosis .. verdict end
    if not malfunctionDetected then 
        malfunctionDetected = device.fail[diagResultName]:malfunctionDetected() 
    end 
end

-- Formatting output---------------------------------------------------------------------------------------------------------------------------------------------

local intro
local finally



if not device.DEBUG then
    if  device.NOTDETECTED then 
        if malfunctionDetected then
            intro = rl.loc('Using the data provided, we can assume the following kind of malfunction:')
            finally = rl.loc('')
            if string.find(device.PROFILE_TYPE, 'HDD') then         
                finally = finally .. rl.loc('For further diagnostics and data recovery we recommend contacting a reputable specialist with professional equipment, preferably PC-3000.')
            end
        else
            intro = rl.loc('Not enough data to determine the malfunction type.')
            if string.find(device.PROFILE_TYPE, 'HDD') then         
                finally = rl.loc('The sounds the hard drive is making are very important for determining the malfunction type. Please pay special attention to that part of the questionnaire.')
            end
        end     
    else
        if malfunctionDetected then
            intro = rl.loc('Following signs of a hardware failure found:')
            finally = rl.loc('For further diagnostics and data recovery we recommend contacting a reputable specialist with professional equipment, preferably PC-3000.')
        else
            if string.find(device.PROFILE_TYPE, 'HDD') and device.FORM_DATA_IMPORTANCE == 'CRITICAL' and device.FORM_MECHANICAL_SHOCK == 'YES' then
                intro = rl.loc('No diagnostics is performed if there is a reason to suspect mechanical failure of a drive with critical data.')
                inally = rl.loc('Heads condition needs to be checked to prevent the drive from destroying its magnetic surface irreversibly. This can be done by an experienced DR specialist only.')
            else
                intro = rl.loc('No signs of hardware faults detected.')
                finally = rl.loc('If the data is not available, this may be a result of file system corruption. You can try using a data recovery software, we recommend Recovery Explorer. Do not run Check Disk, fsck or similar utilities to avoid further data loss.')
            end
        end     
    end
	if device.PROFILE_TYPE == 'FLASHDRIVE' or device.PROFILE_TYPE == 'SDCARD' then
	    finally = finally .. '\n\n' .. rl.loc('disclaimer_flash')
	else
	    finally = finally .. '\n\n' .. rl.loc('disclaimer')
	end
end

if intro and intro ~= '\n' then resultString = resultString .. intro .. '\n' end
if diagnosis and diagnosis ~= '\n' then resultString = resultString .. diagnosis end
if explanation and explanation ~= '\n' then resultString = resultString .. '\n' .. rl.loc('<===< EXPLANATION >===>') .. explanation end
if not malfunctionDetected and warning and warning ~= '\n' then resultString = resultString .. '\n' ..  rl.loc('!!! WARNING !!!') .. warning end
if finally and finally ~= '\n' then resultString = resultString  .. '\n' .. finally .. '\n' end




if device.DBG_CONTAINER then resultString = '+++ CONTAINER VIEWING MODE +++\n' .. resultString .. '\n### STORED INFORMATION ###\n' .. QuickDiagResult end
if device.DEBUG then resultString = '*** DEBUG MODE ***\n' .. resultString .. '\n\n### DEBUG INFO ###\n' .. debugInfo end

return resultString

